/*******************(C) COPYRIGHT 2024 Masses-Chip ******************************
*Project Name        : M9P6362_Usart0.Prj
* File Name          : Usart0_Work.C
* Author             : MASSES CHIP
* Version            : V1.03
* Date               : 2025/04/08
* Web    			 : www.masses-chip.com
* FAE                : Luo
*  				 	 : QQ：411680975
********************************************************************************
*
*Option:16T,WDT-Enable-350ms,RST-IO,BOR 2.4V 
*说明：本范例为异步串口通讯，串口波特率为9600
*中断接收数据放在R_Receive_Data寄存器中
*串口接收错误数据放在R_Receive_Data_Err寄存器中
*串口端口有复用功能，TX端口为IOB0，RX端口为IOB1
******************************************************************************/
//===============================================================================
//*******************************头文件和调用申明********************************
//===============================================================================
//用户不可更改
#include "COMMON.H"
#define UART_BAUDRATE	9600	//波特率，支持9600及以上波特率修改
//===============================================================================
//*********************************Usart0处理************************************
//Usart0处理:串口选择Hex,发送串口所接收到的数据
//===============================================================================
void Usart0_Work(void)
{
	if(TX0IE)
	{
		if(TX0IF)
		{
			TX0REG = R_Receive_Data;
			TX0IE = 0;
		}
	}
}
//===============================================================================
//*****************************Usart_Send处理函数********************************
//发送一个8位数据
//===============================================================================
void Usart_Send_Byte(unsigned char Data)
{
	if(b_sending_flag)
	{
		b_sending_flag = 0;
		TX0REG = Data;
		while(!T0MCLR);
	}
}
//===============================================================================
//********************Usart_Receive中断处理函数**********************************
//Usart 接收数据存入R_Receive_Data寄存器中
//串口接收错误数据放在R_Receive_Data_Err寄存器中
//===============================================================================
void Receive_Interrupt (void)				
{
	if(RX0IF)
	{
		if(FR0ER==0)
		{
			R_Receive_Data = RX0REG; 
			TX0IE = 1;
			b_sending_flag = 1;
		}
		else 
		{
			R_Receive_Data_Err = RX0REG;
		}
	}
}
//===============================================================================
//********************************Usart0初始化***********************************
//目标波特率 = 32M/((BRGD+1)×分频比)
//Usart0异步发送和接收设置
//开启异步接收中断,波特率为9600
//发送数据为8位，TX复用口为IOB0，RX复用口为IOB1
//===============================================================================
void Usart_INIT(void)
{
//1、设置Usart0复用口
//TX复用口为IOB0
//RX复用口为IOB1
	ANSB0 = 0;
	ANSB1 = 0;
	OEB0 = 1;
	PUB0 = 1;
	OEB1 = 0;
	PUB1 = 1;
	MPTX0 = 0x10;
	MPRX0 = 0x11;
//2、Usart发送设置
	TX0CR = 0x80;
//3、波特率设置
	BRGD0H = ((32000000/4/UART_BAUDRATE-1)&0x0300)>>8;
	BRGD0L = ((32000000/4/UART_BAUDRATE-1)&0x00FF);
//4、Usart接收设置
	RX0CR = 0x80;
//5、中断设置
	RX0IF = 0;
	RX0IE = 1;
	RX0IP = 0;
	TX0IF = 0;
	TX0IP = 0;
	GIEH = 1;
}

/*********************************END OF FILE************************************/